<?php

namespace Drp\NativeEventApp\Notifications\Listeners;

use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Notifications\Events\SendNotification;
use Drp\NativeEventApp\Notifications\Events\SendPushNotification;
use Illuminate\Support\Collection;

class NotificationSender
{
    /**
     * @var \Illuminate\Support\Collection
     */
    protected $devices;

    public function __construct()
    {
        $this->devices = collect();
    }

    public function handle(SendNotification $event): void
    {
        if ($event->notification->segments()->exists()) {
            $this->queueSegments(
                $event->notification->segments()
                    ->with('registrations.device')
                    ->get()
            );
        }

        $this->queueDevices($event->notification->devices);

        $this->send($event->notification);
    }

    protected function queueSegments(Collection $segments): void
    {
        $this->devices = $this->devices->merge($segments->flatMap->registrations->map->device);
    }

    protected function queueDevices(Collection $devices): void
    {
        $this->devices = $this->devices->merge($devices);
    }

    protected function send(Notification $notification): void
    {
        $this->devices->unique()->each(static function (Device $device) use ($notification): void {
            event(new SendPushNotification($notification, $device));
        });

        $notification->markAsSent();
    }
}
