<?php

namespace Drp\NativeEventApp\Quiz;

use Drp\NativeEventApp\Models\Quiz;
use Drp\NativeEventApp\Models\QuizQuestion;
use Drp\NativeEventApp\Quiz\Client\QuizClient;
use Drp\NativeEventApp\Quiz\Contracts\QuizOperator;

class ApiQuizOperator implements QuizOperator
{
    /**
     * @var \Drp\NativeEventApp\Quiz\Client\QuizClient
     */
    private $client;

    public function __construct(QuizClient $client)
    {
        $this->client = $client;
    }

    public function create(Quiz $quiz): void
    {
        $this->client->post('', [
            'id' => $quiz->id,
            'question' => null,
        ]);
    }

    public function start(Quiz $quiz): void
    {
        $this->client->post($quiz->id . '/start');
    }

    public function end(Quiz $quiz): void
    {
        $this->client->post($quiz->id . '/end');
    }

    public function delete(Quiz $quiz): void
    {
        $this->client->delete($quiz->id);
    }

    public function startQuestion(Quiz $quiz, QuizQuestion $question): void
    {
        $this->client->post($quiz->id . '/question/' . $question->id . '/start', [
            'question' => $question->id,
        ]);
    }

    public function endQuestion(Quiz $quiz, QuizQuestion $question): void
    {
        $this->client->post($quiz->id . '/question/' . $question->id . '/end');
    }

    public function revealAnswer(Quiz $quiz, QuizQuestion $question): void
    {
        $this->client->post($quiz->id . '/question/' . $question->id . '/end', [
            'answer' => $question->correctAnswers->first()->id,
        ]);
    }
}
