<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;

class HandleFileUploadTest extends TestCase
{
    /** @test */
    public function it_can_store_an_uploaded_file(): void
    {
        Storage::fake(config('native-event-app.files.local.disk'));
        $file = UploadedFile::fake()->image('selfie.jpg');

        $uuid = (string) Str::uuid();
        $url = $this->createSignedUrl($uuid);

        $response = $this->json('PUT', $url, [
            'file' => $file,
        ]);

        $response->assertStatus(200);
        $response->assertJson([
            'id' => $uuid,
        ]);

        Storage::disk(config('native-event-app.files.local.disk'))
            ->assertExists(config('native-event-app.files.local.path') . '/' .$uuid . '/' . $uuid . '.jpg');
    }

    /** @test */
    public function it_expects_a_valid_signature(): void
    {
        $uuid = (string) Str::uuid();
        $url = route('eventpal.sdk.file-upload', [
            'uuid' => $uuid,
            'expires' => 12345,
            'signature' => 'invalid_signature',
        ]);

        Storage::fake(config('native-event-app.files.local.disk'));
        $file = UploadedFile::fake()->image('selfie.jpg');

        $response = $this->json('PUT', $url, [
            'file' => $file,
        ]);

        $response->assertStatus(403);
        Storage::disk(config('native-event-app.files.local.disk'))
            ->assertMissing(config('native-event-app.files.local.path') . '/' . $uuid . '.jpg');
    }

    /** @test */
    public function it_validates_the_request(): void
    {
        $uuid = (string) Str::uuid();
        $response = $this->json('PUT', $this->createSignedUrl($uuid), []);

        $response->assertStatus(422);
        $response->assertJsonValidationErrors(['file']);
        $response->assertJsonMissingValidationErrors(['uuid', 'signature', 'expires']);
    }

    private function createSignedUrl(string $uuid): string
    {
        return URL::temporarySignedRoute(
            'eventpal.sdk.file-upload',
            now()->addMinutes(5),
            ['uuid' => $uuid]
        );
    }
}
