<?php

namespace Drp\NativeEventApp\Test\Unit\FileUploads;

use Drp\NativeEventApp\FileUploads\Contracts\FileUploadDriver;
use Drp\NativeEventApp\Models\FileUpload;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Support\Str;

trait FileUploadContractTest
{
    abstract public function driver(): FileUploadDriver;

    abstract public function storeFile(string $path, string $uuid): void;

    abstract public function assertFileCorrectlyUploaded(FileUpload $fileUpload, string $uuid): void;

    /** @test */
    public function stores_a_file_upload_reference(): void
    {
        $driver = $this->driver();

        $uuid = (string) Str::uuid();
        $this->storeFile(__DIR__ . '/test.png', $uuid);
        $fileUpload = $driver->storeReference($uuid, 1);

        $this->assertEquals($uuid, $fileUpload->uuid);
        $this->assertEquals(1, $fileUpload->registration_id);
        $this->assertFileCorrectlyUploaded($fileUpload, $uuid);
    }

    /** @test */
    public function it_throws_a_file_not_found_exception_on_storing_reference_if_file_doesnt_exist(): void
    {
        $driver = $this->driver();

        $uuid = (string) Str::uuid();

        $this->expectException(FileNotFoundException::class);
        $driver->storeReference($uuid, 1);

        $this->assertEquals(0, FileUpload::count());
    }
}
